/************************************************************************
	sound.c

    Simon18F - An MB Electronics Simon clone based on the PIC18F2550
    Copyright (C) 2009 Simon Inns

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

	Email: simon.inns@gmail.com

************************************************************************/

#include <htc.h>
#include "hardware.h"
#include "sound.h"

// Note: this routine is accurate to about +-5Hz (it gets less accurate the higher
// the frequency)
void soundOn(int frequency)
{
	// Check that we are not already making a sound
	if (hardware.sound.state == ON) return;

	int timerValue;

	// Since the PWM does not allow good low frequency pulse generation
	// when the clock speed is much above 4 Mhz
	// we will use timer1 to generate the PWM by turning the pulse
	// on and off at a 50% duty cycle

	// timer1 must be in high priority mode or other interrupts will cause
	// the sound to 'warble' as they are processed.

	// Timer1 must be called at twice the desired frequency in order to
	// turn the pulse on and off (at 50% duty cycle).  Since the clock
	// is 40Mhz the Fosc/4 is 12,000,000 timer ticks per second, so the required
	// timer value is 6,000,000 ticks / frequency

	// By using a prescaler of 1:4 we can reduce the figure to
	// 1,500,000 ticks / frequency

	// Since the timer is 16-bit the maximum timer setting is 65535, meaning that
	// the lowest frequency we can generate is around 23 Hz
	// however we limit the range to 30-20,000 Hz to be on the safe-side

	// Sanity check the frequency
	if (frequency < 20) frequency = 30;
	if (frequency > 20000) frequency = 200000;

	timerValue = 65536 - (1500000 / frequency);

	// Set the TMR1L and TMR1H bytes according to timerValue
	hardware.sound.timer1HighByte = (char)(timerValue >> 8);
	hardware.sound.timer1LowByte =  (char)timerValue;

	TMR1H = hardware.sound.timer1HighByte;
	TMR1L = hardware.sound.timer1LowByte;

	// Set the PWM sound state to ON (used by the interrupt to track if the pulse
	// should be high or low)
	hardware.sound.pwmState = ON;

	// Set the sound state to ON
	hardware.sound.state = ON;

	// Set up the timer
	TMR1IF = 0; // Reset the timer1 interrupt flag
	T1CON = 0b10100001;	// Timer1 on, 16-bit and 1:4 prescaler
	TMR1IE = 1; // Turn on the timer1 interrupt

}

void soundOff(void)
{
	// This routine stops the sound by disabling the timer1 interrupt
	TMR1IE = 0;
	T1CON = 0b10100000;	// Timer1 off, 16-bit and 1:4 prescaler

	// Set the sound state to OFF
	hardware.sound.state = OFF;

	// Make sure the port is low
	SPEAKER = OFF;
}

// Utility routine to make it easy to play common game sounds
void gameSound(unsigned char sound)
{
	switch(sound)
	{
		case GREEN:		soundOn(GREEN_SOUND);
						break;
		case RED:		soundOn(RED_SOUND);
						break;
		case YELLOW:	soundOn(YELLOW_SOUND);
						break;
		case BLUE:		soundOn(BLUE_SOUND);
						break;
		case FAIL:		soundOn(FAIL_SOUND);
						break;
	}
}